<?php
require_once('vendor/flexi/flexi.php');
require_once 'lib/include/reiter.inc.php';
class BenutzerStatistikPlugin extends AbstractStudIPSystemPlugin
{
	public $user;
	private $permissions;
	private $template_factory;
	private $db;
	private $tabs = array(
		'show'=>'Gesamt',
		'today'=>'Heute',
		'yesterday'=>'Gestern',
		'monthly'=>'Monatlich',
		'quarterly'=>'Quartalsweise'
	);

	public function BenutzerStatistikPlugin()
	{
		parent::AbstractStudIPSystemPlugin();

		$this->user = $this->getUser();
		$this->permissions = $this->user->getPermission();
		if ($this->permissions->hasRootPermission())
		{
			$nav = new PluginNavigation();
			$nav->setDisplayname($this->getPluginname());
			$this->setNavigation($nav);

			$this->setPluginiconname('images/stats.gif');
		}
		$this->template_factory = new Flexi_TemplateFactory(dirname(__FILE__).'/templates');
		$this->db = new DB_Seminar;

		$path = str_replace($GLOBALS['ABSOLUTE_PATH_STUDIP'], '', dirname(__FILE__));
		$GLOBALS['_include_additional_header'] .= '<link href="'.$path.'/css/statistics.css" type="text/css" rel="stylesheet"/>'."\n";
		$GLOBALS['_include_additional_header'] .= '<script type="text/javascript" src="'.$path.'/js/javascript.js"></script>'."\n";
	}

	public function getPluginname()
	{
		return _('Benutzer-Statistik');
	}

	public function hasBackgroundTasks()
	{
		return true;
	}

	public function doBackgroundTasks()
	{
		static $hit_once = false;

		if (!empty($_POST['sniffer_ping']))
		{
			$this->StoreHit();
		}
		elseif ($hit_once)
		{
			return;
		}

		$user_agent = empty($_SERVER['HTTP_USER_AGENT']) ? '?' : $_SERVER['HTTP_USER_AGENT'];

		$permission = 'unknown';
		if ($this->permissions->hasAdminPermission() or $this->permissions->hasRootPermission())
		{
			$permission = 'admin';
		}
		elseif ($this->permissions->hasTeacherPermission())
		{
			$permission = 'teacher';
		}
		elseif ($this->permissions->hasTutorPermission() or $this->permissions->hasStudentPermission())
		{
			$permission = 'student';
		}

		$this->db->queryf("INSERT INTO user_statistics (ip, user_agent, daystamp, hash, permission) VALUES (INET_ATON('%s'), '%s', NOW(), MD5(CONCAT('%s', '%s')), '%s') ON DUPLICATE KEY UPDATE hits=hits+1", $_SERVER['REMOTE_ADDR'], $user_agent, $this->user->getUserid(), $this->user->getUsername(), $permission);

		$this->db->queryf("SELECT COUNT(*) AS quantity FROM user_statistics WHERE ip=INET_ATON('%s') AND user_agent='%s' AND daystamp=NOW() AND hash=MD5(CONCAT('%s', '%s')) AND (hits>5 OR javascript=1)", $_SERVER['REMOTE_ADDR'], $user_agent, $this->user->getUserid(), $this->user->getUsername());
		if (!$this->db->next_record() or $this->db->f('quantity')==0)
		{
			$uri = 'http'.($_SERVER['SERVER_PORT']!=80?'s':'').'://'.$_SERVER['HTTP_HOST'].dirname($_SERVER['REQUEST_URI']).'/index.php';

			$template = $this->template_factory->open('sniffer');
			$template->set_attribute('link', $uri);
			echo $template->render();
		}
		$this->db->free();

		$hit_once = true;
	}

	private function StoreHit()
	{
		ob_end_clean();

		$user_agent = empty($_SERVER['HTTP_USER_AGENT']) ? '?' : $_SERVER['HTTP_USER_AGENT'];
		$this->db->queryf("UPDATE user_statistics SET javascript=1, screen_width=%d, screen_height=%d WHERE ip=INET_ATON('%s') AND user_agent='%s' AND hash=MD5(CONCAT('%s', '%s')) AND daystamp=NOW()", $_POST['width'], $_POST['height'], $_SERVER['REMOTE_ADDR'], $user_agent, $this->user->getUserid(), $this->user->getUsername());

		die;
	}

	private function displayHeader()
	{
		if (!$this->permissions->hasRootPermission())
		{
			echo _('Unberechtigter Zugriff');
			page_close();
			die;
		}

		$structure = array();
		foreach ($this->tabs as $cmd=>$title)
		{
			$structure[$cmd] = array(
				'topKat'=>'',
				'name'=>_($title),
				'link'=>PluginEngine::getLink($this, array(), $cmd),
				'active'=>($cmd==$_REQUEST['cmd'])
			);
		}

		$reiter = new reiter;
		$reiter->create($structure, $_REQUEST['cmd']);
		echo '<br/>';
	}

	public function actionshow()
	{
		$this->displayStatistics();
	}

	public function actiontoday()
	{
		$this->displayStatistics(time());
	}

	public function actionyesterday()
	{
		$this->displayStatistics(strtotime('yesterday'));
	}

	private function displayStatistics($timestamp=null)
	{
		$this->displayHeader();

		$template = $this->template_factory->open('stats');
		$template->set_attribute('stats', $this->getStatistics($timestamp));
		$template->set_attribute('link', PluginEngine::getLink($this, array()).'&agent=');
		$template->set_attribute('icon_path', str_replace($GLOBALS['ABSOLUTE_PATH_STUDIP'], '', dirname(__FILE__)).'/images/');
		echo $template->render();
	}

	public function actionmonthly()
	{
		$month_names = array(
			1=>_('Januar'),
			2=>_('Februar'),
			3=>_('Mrz'),
			4=>_('April'),
			5=>_('Mai'),
			6=>_('Juni'),
			7=>_('Juli'),
			8=>_('August'),
			9=>_('September'),
			10=>_('Oktober'),
			11=>_('November'),
			12=>_('Dezember')
		);

		$this->displayHeader();

		if (empty($_REQUEST['month']))
		{
			$month = date('n');
			$year = date('Y');
		}
		else
		{
			list($year, $month) = explode('-', $_REQUEST['month']);
		}

		$months = array();
		$this->db->query("SELECT DISTINCT DATE_FORMAT(daystamp, '%Y-%c') AS month_stamp FROM user_statistics ORDER BY month_stamp DESC");
		while ($this->db->next_record())
		{
			list($y, $m) = explode('-', $this->db->f('month_stamp'));
			$months[$this->db->f('month_stamp')] = $month_names[$m].' '.$y;
		}
		$this->db->free();

		$stats = array();
		$this->db->queryf("SELECT DATE_FORMAT(daystamp, '%%e') AS day, SUM(hits) AS hits, COUNT(hash) AS visits, COUNT(DISTINCT hash) AS headcount FROM user_statistics WHERE daystamp BETWEEN FROM_UNIXTIME(%d) AND FROM_UNIXTIME(%d) GROUP BY daystamp", mktime(0,0,0,$month,1,$year), mktime(0,0,0,$month+1,1,$year)-1);
		while ($this->db->next_record())
		{
			$stats[$this->db->f('day')] = array(
				'hits'=>$this->db->f('hits'),
				'visits'=>$this->db->f('visits'),
				'headcount'=>$this->db->f('headcount')
			);
		}
		$this->db->free();

		$max = array();
		$totals = array();
		$average = array(
			'visits'=>0,
			'headcount'=>0,
			'hits'=>0
		);
		$average_count = 0;
		foreach ($stats as $stat)
		{
			foreach ($stat as $index=>$value)
			{
				if (empty($totals[$index]))
				{
					$totals[$index] = 0;
				}
				$totals[$index] += $value;

				if (empty($max[$index]))
				{
					$max[$index] = 0;
				}
				$max[$index] = max($max[$index], $value);

				$average[$index] += $value;
			}
			$average_count++;
		}
		foreach ($average as $index=>$value)
		{
			$average[$index] = $average_count>0 ? $value/$average_count : 0;
		}

		$this->db->queryf("SELECT COUNT(DISTINCT hash) AS headcount FROM user_statistics WHERE daystamp BETWEEN FROM_UNIXTIME(%d) AND FROM_UNIXTIME(%d)", mktime(0,0,0,$month,1,$year), mktime(0,0,0,$month+1,1,$year)-1);
		$totals['headcount'] = $this->db->next_record() ? $this->db->f('headcount') : '?';
		$this->db->free();

		$template = $this->template_factory->open('monthly');
		$template->set_attribute('type', empty($_REQUEST['type']) ? 'visits' : $_REQUEST['type']);
		$template->set_attribute('months', $months);
		$template->set_attribute('month_stamp', mktime(0,0,0, $month, 1, $year));
		$template->set_attribute('stats', $stats);
		$template->set_attribute('max', $max);
		$template->set_attribute('totals', $totals);
		$template->set_attribute('average', $average);
		$template->set_attribute('visits_scale', $this->getScale($max['visits']));
		$template->set_attribute('hits_scale', $this->getScale($max['hits']));
		echo $template->render();
	}

	public function actionquarterly()
	{
		$this->displayHeader();

		$quarters = array();

		$this->db->queryf("SELECT DISTINCT CONCAT(DATE_FORMAT(daystamp, '%%Y'),'-',CEIL(DATE_FORMAT(daystamp, '%%m')/3)) AS quarter_stamp FROM user_statistics u ORDER BY quarter_stamp ASC");
		while ($this->db->next_record())
		{
			list($year, $quarter) = explode('-', $this->db->f('quarter_stamp'));
			$quarters[$this->db->f('quarter_stamp')] = array(
				'quarter'=>$quarter,
				'year'=>$year
			);
		}
		$this->db->free();

		if (empty($_REQUEST['quarter']))
		{
			$quarter = ceil(date('m')/3);
			$year = date('Y');
		}
		else
		{
			list ($year, $quarter) = explode('-', $_REQUEST['quarter']);
		}
		$quarter_stamp = mktime(0,0,0, $quarter*3-2, 1, $year);

		$stats = array();
		$this->db->queryf("SELECT DATE_FORMAT(daystamp, '%%c-%%e') AS day, SUM(hits) AS hits, COUNT(hash) AS visits, COUNT(DISTINCT hash) AS headcount FROM user_statistics WHERE daystamp BETWEEN FROM_UNIXTIME(%d) AND FROM_UNIXTIME(%d) GROUP BY daystamp", $quarter_stamp, strtotime('3 months', $quarter_stamp)-1);
		while ($this->db->next_record())
		{
			$stats[$this->db->f('day')] = array(
				'hits'=>$this->db->f('hits'),
				'visits'=>$this->db->f('visits'),
				'headcount'=>$this->db->f('headcount')
			);
		}
		$this->db->free();

		$max = array();
		$totals = array();
		$average = array(
			'visits'=>0,
			'headcount'=>0,
			'hits'=>0
		);
		$average_count = 0;
		foreach ($stats as $stat)
		{
			foreach ($stat as $index=>$value)
			{
				if (empty($totals[$index]))
				{
					$totals[$index] = 0;
				}
				$totals[$index] += $value;

				if (empty($max[$index]))
				{
					$max[$index] = 0;
				}
				$max[$index] = max($max[$index], $value);

				$average[$index] += $value;
			}
			$average_count++;
		}
		foreach ($average as $index=>$value)
		{
			$average[$index] = $value/$average_count;
		}

		$this->db->queryf("SELECT COUNT(DISTINCT hash) AS headcount FROM user_statistics WHERE daystamp BETWEEN FROM_UNIXTIME(%d) AND FROM_UNIXTIME(%d)", $quarter_stamp, strtotime('3 months', $quarter_stamp)-1);
		$totals['headcount'] = $this->db->next_record() ? $this->db->f('headcount') : '?';
		$this->db->free();

		$days = date('t', $quarter_stamp)+date('t', strtotime('1 month', $quarter_stamp))+date('t', strtotime('2 months', $quarter_stamp));

		$day_captions = array();
		$current_caption = array(
			'title'=>'1',
			'span'=>1
		);
		for ($i=1, $stamp=strtotime('tomorrow', $quarter_stamp); $i<$days; $i++, $stamp=strtotime('tomorrow', $stamp))
		{
			$day = date('j', $stamp);
			if ($day%5==0 or $day==1)
			{
				array_push($day_captions, $current_caption);
				$current_caption = array(
					'title'=>date('j', $stamp),
					'span'=>0
				);
			}
			$current_caption['span']+=1;
		}
		if ($current_caption['span']!=0)
		{
			array_push($day_captions, $current_caption);
		}

		$template = $this->template_factory->open('quarterly');
		$template->set_attribute('type', empty($_REQUEST['type']) ? 'visits' : $_REQUEST['type']);
		$template->set_attribute('links', array(
			'submit'=>PluginEngine::getLink($this, array(), 'quarterly'),
			'monthly'=>PluginEngine::getLink($this, array(), 'monthly')
		));
		$template->set_attribute('quarters', $quarters);
		$template->set_attribute('quarter_stamp', $quarter_stamp);
		$template->set_attribute('days', $days);
		$template->set_attribute('td_width', round(95/$days, 2));
		$template->set_attribute('stats', $stats);
		$template->set_attribute('totals', $totals);
		$template->set_attribute('average', $average);
		$template->set_attribute('max', $max);
		$template->set_attribute('day_captions', $day_captions);
		$template->set_attribute('visits_scale', $this->getScale($max['visits']));
		$template->set_attribute('hits_scale', $this->getScale($max['hits']));
		$template->set_attribute('months', array(
			1=>_('Januar'),
			2=>_('Februar'),
			3=>_('Mrz'),
			4=>_('April'),
			5=>_('Mai'),
			6=>_('Juni'),
			7=>_('Juli'),
			8=>_('August'),
			9=>_('September'),
			10=>_('Oktober'),
			11=>_('November'),
			12=>_('Dezember')
		));
		echo $template->render();
	}

	private function getScale($max)
	{
		$possible_scales = array(10, 25, 50);
		$test_scales = $possible_scales;

		do
		{
			$test_scale = array_shift($test_scales);
			if (empty($test_scales))
			{
				foreach ($possible_scales as $index=>$scale)
				{
					$possible_scales[$index] = $scale*10;
				}
				$test_scales = $possible_scales;
			}
		}
		while ($max/$test_scale>10);

		$scale = array();
		$current_scale = 0;
		while ($current_scale<$max)
		{
			array_push($scale, array(
				'value'=>$current_scale,
				'percent'=>$current_scale/$max
			));
			$current_scale += $test_scale;
		}

		return $scale;
	}

	private function getStatistics($date=null, $end_date=null)
	{
		$query_append = '';
		if ($date!==null and $end_date===null)
		{
			$query_append .= " daystamp=DATE(FROM_UNIXTIME(".$date."))";
		}
		elseif ($date!==null and $end_date!==null)
		{
			$query_append .= " daystamp BETWEEN DATE(FROM_UNIXTIME(".$date.")) AND DATE(FROM_UNIXTIME(".$end_date."))";
		}

		$query = "SELECT COUNT(DISTINCT hash) AS headcount, DATE_FORMAT(MIN(daystamp), '%%d.%%m.%%Y') AS start_date, DATE_FORMAT(MAX(daystamp), '%%d.%%m.%%Y') AS end_date, COUNT(*) AS visits, SUM(hits) AS hits, SUM(javascript)*100/COUNT(*) AS javascript_percentage, AVG(hits) AS avg_hits FROM user_statistics";
		if (!empty($query_append))
		{
			$query .= " WHERE".$query_append;
		}
		$this->db->queryf($query);
		$this->db->next_record();
		$stats = array(
			'date'=> ($date!==null and $end_date===null)
				? date('d.m.Y', $date)
				: array(
					'start'=>$this->db->f('start_date'),
					'end'=>$this->db->f('end_date')
				)
			,
			'visits'=>$this->db->f('visits'),
			'headcount'=>$this->db->f('headcount'),
			'hits'=>$this->db->f('hits'),
			'avg_hits'=> $this->db->f('avg_hits'),
			'javascript_percentage'=>$this->db->f('javascript_percentage'),
			'screen_sizes'=>array(),
			'user_agents'=>array(),
			'permissions'=>array()
		);
		$this->db->free();

		$query = "SELECT COUNT(*) AS quantity, COUNT(DISTINCT hash) AS headcount, permission FROM user_statistics";
		if (!empty($query_append))
		{
			$query .= " WHERE".$query_append;
		}
		$query .= " GROUP BY permission";
		$this->db->queryf($query);
		while ($this->db->next_record())
		{
			$stats['permissions'][$this->db->f('permission')] = $this->db->f('quantity');
			$stats['permissions'][$this->db->f('permission').'_headcount'] = $this->db->f('headcount');
		}
		$this->db->free();

		$query = "SELECT COUNT(*) AS quantity, screen_width, screen_height FROM user_statistics WHERE screen_width IS NOT NULL AND screen_height IS NOT NULL ".(empty($query_append)?'':" AND".$query_append)."GROUP BY screen_width, screen_height ORDER BY quantity DESC LIMIT 10";
		$this->db->queryf($query);
		while ($this->db->next_record())
		{
			array_push($stats['screen_sizes'], array(
				'width'=>$this->db->f('screen_width'),
				'height'=>$this->db->f('screen_height'),
				'percentage'=>$this->db->f('quantity')*100/$stats['visits']
			));
		}
		$this->db->free();

		$query = "SELECT user_agent, COUNT(*) AS quantity FROM user_statistics ".(empty($query_append)?'':" WHERE".$query_append)."GROUP BY user_agent";
		$this->db->queryf($query);
		$user_agents = array();
		while ($this->db->next_record())
		{
			array_push($user_agents, array(
				'user_agent'=>$this->db->f('user_agent'),
				'quantity'=>$this->db->f('quantity')
			));
		}
		$this->db->free();

		$stats['user_agents'] = $this->parseUserAgents($user_agents);
		$stats['os'] = $this->parseOS($user_agents);

		return $stats;
	}

	private function parseOS($user_agents)
	{
		$possible_os = array(
			'Windows'=>'Windows',
			'Macintosh'=>'Macintosh',
			'Linux'=>'X11'
		);

		$result = array();
		foreach ($user_agents as $user_agent)
		{
			$quantity = $user_agent['quantity'];
			$os = _('unbekannt');
			foreach ($possible_os as $os_name=>$os2try)
			{
				if (strpos($user_agent['user_agent'], $os2try)!==false)
				{
					$os = $os_name;
					break;
				}
			}

			if (!empty($result[$os]))
			{
				$result[$os]['quantity'] += $quantity;
			}
			else
			{
				$result[$os] = array(
					'name'=>$os,
					'quantity'=>$quantity
				);
			}
		}
		usort($result, create_function('$a, $b', 'return $b["quantity"]-$a["quantity"];'));

		return array_slice($result, 0, 10);
	}

	private function parseUserAgents($user_agents)
	{
		$result = array();
		foreach ($user_agents as $user_agent)
		{
			$quantity = $user_agent['quantity'];
			$agent = $user_agent['user_agent'];
			$version = '?';
			if (preg_match('~(\d+(?:\.\d+)*)\s+(Safari)~i', $agent, $matches))
			{
				$agent = $matches[2];
				$version = $matches[1];
			}
			elseif (preg_match('~(Epiphany|Camino|Safari|SeaMonkey|IceWeasel|Konqueror|Netscape|Opera|Firefox|MSIE)[ /]([^;\s]+)(?:;|\s|$)~i', $agent, $matches))
			{
				$agent = $matches[1];
				$version = $matches[2];
			}
			else
			{
				$version = $agent;
				$agent = _('unbekannt');
			}

			$hash = md5($agent);
			if (!empty($result[$hash]))
			{
				$result[$hash]['quantity'] += $quantity;
				if (empty($result[$hash]['versions'][$version]))
				{
					$result[$hash]['versions'][$version] = 0;
				}
				$result[$hash]['versions'][$version] += $quantity;
			}
			else
			{
				$result[$hash] = array(
					'name'=>$agent,
					'versions'=>array(
						$version=>$quantity
					),
					'quantity'=>$quantity
				);
			}
		}

		foreach ($result as $index=>$item)
		{
			uasort($item['versions'], create_function('$a, $b', 'return $b-$a;'));
			$result[$index] = $item;
		}
		usort($result, create_function('$a, $b', 'return $b["quantity"]-$a["quantity"];'));

		return array_slice($result, 0, 10);
	}

	public function show()
	{
		if (!$this->permissions->hasRootPermission())
		{
			echo _('Ungltiger Zugriff');
		}
		return $this->actionshow();
	}

	public function today()
	{
		return $this->actiontoday();
	}

	public function yesterday()
	{
		return $this->actionyesterday();
	}

	public function monthly()
	{
		return $this->actionmonthly();
	}
}
?>